package com.vaadin.spring.roo.addon;

import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.logging.Logger;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;
import org.osgi.service.component.ComponentContext;
import org.springframework.roo.addon.entity.EntityMetadata;
import org.springframework.roo.classpath.PhysicalTypeMetadata;
import org.springframework.roo.classpath.PhysicalTypeMetadataProvider;
import org.springframework.roo.classpath.details.ClassOrInterfaceTypeDetails;
import org.springframework.roo.classpath.itd.ItdMetadataScanner;
import org.springframework.roo.classpath.operations.ClasspathOperations;
import org.springframework.roo.file.monitor.event.FileDetails;
import org.springframework.roo.metadata.MetadataDependencyRegistry;
import org.springframework.roo.metadata.MetadataItem;
import org.springframework.roo.metadata.MetadataService;
import org.springframework.roo.model.JavaPackage;
import org.springframework.roo.model.JavaSymbolName;
import org.springframework.roo.model.JavaType;
import org.springframework.roo.process.manager.FileManager;
import org.springframework.roo.project.Dependency;
import org.springframework.roo.project.Path;
import org.springframework.roo.project.PathResolver;
import org.springframework.roo.project.ProjectMetadata;
import org.springframework.roo.project.ProjectOperations;
import org.springframework.roo.project.ProjectType;
import org.springframework.roo.project.Repository;
import org.springframework.roo.support.util.Assert;
import org.springframework.roo.support.util.StringUtils;
import org.springframework.roo.support.util.TemplateUtils;
import org.springframework.roo.support.util.WebXmlUtils;
import org.springframework.roo.support.util.XmlUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.vaadin.spring.roo.addon.abstractentityview.VaadinAbstractEntityViewMetadata;
import com.vaadin.spring.roo.addon.annotations.RooVaadinAbstractEntityView;
import com.vaadin.spring.roo.addon.entityview.VaadinEntityViewMetadata;

/**
 * Implementation of Vaadin Roo commands that are available via the Roo shell.
 *
 * @since 1.1.0M1
 */
@Component
@Service
public class VaadinOperationsImpl implements VaadinOperations {

	private static final String ABSTRACT_ENTITY_VIEW_CLASS = "AbstractEntityView";
	private static final String AUTOMATIC_ENTITY_FORM_CLASS = "AutomaticEntityForm";

	private static final String VAADIN_ARTIFACT_ID = "vaadin";
	private static final String VAADIN_GROUP_ID = "com.vaadin";

	private static final String OPEN_ENTITYMANAGER_IN_VIEW_FILTER_NAME = "Spring OpenEntityManagerInViewFilter";

	private static final String ENTITY_FIELD_WRAPPER_CLASS = "EntityFieldWrapper";
	private static final String ENTITY_SET_FIELD_WRAPPER_CLASS = "EntitySetFieldWrapper";
	private static final String ENTITY_TABLE_COLUMN_GENERATOR_CLASS = "EntityTableColumnGenerator";

	private static final String ENTITY_EDITOR_INTERFACE = "EntityEditor";

	private static Logger logger = Logger.getLogger(VaadinOperations.class.getName());

	@Reference
	private FileManager fileManager;
	@Reference
	private PathResolver pathResolver;
	@Reference
	private MetadataService metadataService;
	@Reference
	private ProjectOperations projectOperations;
	@Reference
	private ClasspathOperations classpathOperations;
	@Reference
	private PhysicalTypeMetadataProvider physicalTypeMetadataProvider;
	@Reference
	private ItdMetadataScanner itdMetadataScanner;
	@Reference
	private MetadataDependencyRegistry dependencyRegistry;

    private ComponentContext context;

    protected void activate(ComponentContext context) {
	    this.context = context;
    }

	public boolean isProjectAvailable() {
		ProjectMetadata projectMetadata = (ProjectMetadata) metadataService
				.get(ProjectMetadata.getProjectIdentifier());
		return projectMetadata != null;
	}

	public boolean isVaadinSetup() {
		ProjectMetadata projectMetadata = (ProjectMetadata) metadataService
				.get(ProjectMetadata.getProjectIdentifier());
		Set<Dependency> dependencies = projectMetadata.getDependencies();
		for (Dependency d : dependencies) {
			if (VAADIN_ARTIFACT_ID.equals(d.getArtifactId().getSymbolName())
					&& VAADIN_GROUP_ID.equals(d.getGroupId()
							.getFullyQualifiedPackageName())) {
				return true;
			}
		}
		return false;
	}

	public boolean isPersistenceSetup() {
		return pathResolver != null
				&& fileManager.exists(pathResolver.getIdentifier(
						Path.SRC_MAIN_RESOURCES, "META-INF/persistence.xml"));
	}

	public void vaadinSetup(JavaPackage applicationPackage,
			JavaSymbolName baseName, JavaSymbolName themeName,
			boolean useJpaContainer) {
		logger.info("Performing vaadin setup");

		// Parse the configuration.xml file
		Element configuration = XmlUtils.getConfiguration(getClass());

		addVaadinDependencies(configuration);

		if (applicationPackage != null) {

			String pkg = applicationPackage.getFullyQualifiedPackageName();

			String baseNameString;
			if (baseName == null) {
				baseNameString = generateApplicationBaseName(applicationPackage);
			} else {
				baseNameString = baseName.getSymbolNameCapitalisedFirstLetter();
			}

			String themeNameString;
			if (themeName != null) {
				themeNameString = themeName.getSymbolName();
			} else {
				themeNameString = baseNameString.toLowerCase();
			}

			// create theme
			createTheme(themeNameString);

			// create/modify web.xml
			copyWebXml();
			manageWebXml(pkg + "." + getApplicationClassName(baseNameString));

			// make this a WAR project
			projectOperations.updateProjectType(ProjectType.WAR);

			// create a Vaadin application and any related classes
			createApplication(applicationPackage, baseNameString,
					themeNameString, useJpaContainer);
		}
	}

	private void addVaadinDependencies(Element configuration) {
		List<Element> vaadinDependencies = XmlUtils.findElements(
				"/configuration/vaadin/dependencies/dependency", configuration);
		for (Element dependencyElement : vaadinDependencies) {
			projectOperations
					.dependencyUpdate(new Dependency(dependencyElement));
		}
		List<Element> vaadinRepositories = XmlUtils.findElements(
				"/configuration/vaadin/repositories/repository", configuration);
		for (Element repositoryElement : vaadinRepositories) {
			projectOperations.addRepository(new Repository(repositoryElement));
		}
	}

	private void addBeanValidationDependencies(Element configuration) {
		List<Element> beanValidationDependencies = XmlUtils.findElements(
				"/configuration/beanvalidation/dependencies/dependency",
				configuration);
		for (Element dependencyElement : beanValidationDependencies) {
			projectOperations
					.dependencyUpdate(new Dependency(dependencyElement));
		}
	}

	private static void addJpaContainerDependencies(Element configuration, ProjectOperations projectOperations) {
		List<Element> jpaContainerDependencies = XmlUtils.findElements(
				"/configuration/jpacontainer/dependencies/dependency",
				configuration);
		for (Element dependencyElement : jpaContainerDependencies) {
			projectOperations
					.dependencyUpdate(new Dependency(dependencyElement));
		}
	}

	private void createTheme(String themeName) {
		Assert.notNull(pathResolver, "Path resolver not found");

		String themePath = pathResolver.getIdentifier(Path.SRC_MAIN_WEBAPP,
				"VAADIN/themes/" + themeName);
		if (!fileManager.exists(themePath)) {
			VaadinRooUtils.copyDirectoryContents(fileManager, context, "theme",
					themePath);
			VaadinRooUtils.copyDirectoryContents(fileManager, context,
					"theme/img", themePath + "/img");
			VaadinRooUtils.copyDirectoryContents(fileManager, context,
					"theme/entityviews", themePath + "/entityviews");
		} else {
			logger.info("Theme " + themeName + " already exists");
		}
	}

	private void copyWebXml() {
		ProjectMetadata projectMetadata = (ProjectMetadata) metadataService
				.get(ProjectMetadata.getProjectIdentifier());
		Assert.notNull(projectMetadata, "Project metadata required");
		Assert.notNull(pathResolver, "Path resolver required");

		if (fileManager.exists(pathResolver.getIdentifier(Path.SRC_MAIN_WEBAPP,
				"WEB-INF/web.xml"))) {
			// file exists, so nothing to do
			return;
		}

		InputStream templateInputStream = TemplateUtils.getTemplate(getClass(),
				"web-template.xml");
		Document webXml;
		try {
			webXml = XmlUtils.getDocumentBuilder().parse(templateInputStream);
		} catch (Exception ex) {
			throw new IllegalStateException(ex);
		}

		WebXmlUtils.setDisplayName(projectMetadata.getProjectName(), webXml,
				null);
		WebXmlUtils.setDescription(
				"Roo generated " + projectMetadata.getProjectName()
						+ " application", webXml, null);

		VaadinRooUtils
				.writeXmlToDiskIfNecessary(fileManager,
						pathResolver.getIdentifier(Path.SRC_MAIN_WEBAPP,
								"WEB-INF/web.xml"), webXml);

		fileManager.scan();
	}

	private void manageWebXml(String applicationName) {
		ProjectMetadata projectMetadata = (ProjectMetadata) metadataService
				.get(ProjectMetadata.getProjectIdentifier());
		Assert.notNull(projectMetadata, "Project metadata required");
		Assert.notNull(pathResolver, "Path resolver required");

		// Verify that the web.xml already exists
		String webXmlFile = pathResolver.getIdentifier(
				Path.SRC_MAIN_WEBAPP, "WEB-INF/web.xml");
		Assert.isTrue(fileManager.exists(webXmlFile), "'" + webXmlFile
				+ "' does not exist");

		Document webXml;
		try {
			webXml = XmlUtils.getDocumentBuilder().parse(
					fileManager.getInputStream(webXmlFile));
		} catch (Exception ex) {
			throw new IllegalStateException(ex);
		}

		WebXmlUtils.addContextParam(new WebXmlUtils.WebXmlParam(
				"productionMode", "false"), webXml, "Vaadin production mode");

		// can have multiple application context files separated by spaces
		WebXmlUtils.addContextParam(new WebXmlUtils.WebXmlParam(
				"contextConfigLocation",
				"classpath*:META-INF/spring/applicationContext*.xml"), webXml,
				null);

		// optionally make the EntityManager available in each request
		if (fileManager.exists(pathResolver.getIdentifier(
				Path.SRC_MAIN_RESOURCES, "META-INF/persistence.xml"))) {
			WebXmlUtils
					.addFilterAtPosition(
							WebXmlUtils.FilterPosition.FIRST,
							null,
							null,
							OPEN_ENTITYMANAGER_IN_VIEW_FILTER_NAME,
							"org.springframework.orm.jpa.support.OpenEntityManagerInViewFilter",
							"/*", webXml, null);
		}

		// load the Spring context from the appropriate file
		WebXmlUtils
				.addListener(
						"org.springframework.web.context.ContextLoaderListener",
						webXml,
						"Creates the Spring Container shared by all Servlets and Filters");

		// for now, register servlet for /*
		WebXmlUtils.addServlet(projectMetadata.getProjectName()
				+ " Vaadin Application Servlet",
				"com.vaadin.terminal.gwt.server.ApplicationServlet", "/*",
				null, webXml, null, new WebXmlUtils.WebXmlParam("application",
						applicationName));

		VaadinRooUtils.writeXmlToDiskIfNecessary(fileManager, webXmlFile,
				webXml);
	}

	public void createApplication(JavaPackage applicationPackage,
			String baseName, String themeName, boolean useJpaContainer) {
		Assert.notNull(applicationPackage, "Application package required");
		Assert.notNull(classpathOperations, "Class path operations required");
		Assert.notNull(pathResolver, "Path resolver required");

		String pkg = applicationPackage.getFullyQualifiedPackageName();

		String applicationName = getApplicationClassName(baseName);
		String windowName = baseName + "Window";
		String viewName = baseName + "EntityManagerView";
		String abstractEntityViewName = ABSTRACT_ENTITY_VIEW_CLASS;

		String annotationParameters = "";
		if (useJpaContainer) {
			annotationParameters = "useJpaContainer = true";
		}

		Map<String, String> substitutions = new HashMap<String, String>();
		substitutions.put("__TOP_LEVEL_PACKAGE__", pkg);
		substitutions.put("__APPLICATION_CLASS_NAME__", applicationName);
		substitutions.put("__WINDOW_CLASS_NAME__", windowName);
		substitutions.put("__VIEW_CLASS_NAME__", viewName);
		substitutions.put("__THEME_NAME__", themeName);
		substitutions.put("__ABSTRACT_ENTITY_VIEW_PACKAGE__", pkg);
		substitutions.put("__ABSTRACT_ENTITY_VIEW_CLASS__",
				abstractEntityViewName);
		substitutions.put("__ABSTRACT_ENTITY_VIEW_ANNOTATION_PARAMETERS__",
				annotationParameters);

		createClassFromTemplate(pkg + "." + applicationName,
				"Application.java", substitutions);
		createClassFromTemplate(pkg + "." + windowName, "Window.java",
				substitutions);
		createClassFromTemplate(pkg + "." + viewName,
				"EntityManagerView.java", substitutions);
		createClassFromTemplate(pkg + "." + abstractEntityViewName,
				"AbstractEntityView.java", substitutions);
		createClassFromTemplate(pkg + "." + AUTOMATIC_ENTITY_FORM_CLASS,
				"AutomaticEntityForm.java", substitutions);

		// field wrapper classes to convert between identifiers and entities
		createClassFromTemplate(pkg + "." + ENTITY_FIELD_WRAPPER_CLASS,
				"EntityFieldWrapper.java", substitutions);
		createClassFromTemplate(pkg + "." + ENTITY_SET_FIELD_WRAPPER_CLASS,
				"EntitySetFieldWrapper.java", substitutions);

		// UI table column generator
		createClassFromTemplate(
				pkg + "." + ENTITY_TABLE_COLUMN_GENERATOR_CLASS,
				"EntityTableColumnGenerator.java", substitutions);

		// entity editor interface
		createClassFromTemplate(pkg + "." + ENTITY_EDITOR_INTERFACE,
				"EntityEditor.java", substitutions);
	}

	private void createClassFromTemplate(String className,
			String templateBaseName, Map<String, String> substitutions) {
		JavaType classToCreate = new JavaType(className);
		String classResourceIdentifier = classpathOperations
				.getPhysicalLocationCanonicalPath(classToCreate,
						Path.SRC_MAIN_JAVA);
		VaadinRooUtils.installFromTemplateIfNeeded(fileManager,
				classResourceIdentifier, templateBaseName, substitutions);
	}

	private String getApplicationClassName(String baseName) {
		return baseName + "Application";
	}

	/**
	 * Generates a base name (beginning of the class name) for classes to be
	 * created based on the last part of a package path.
	 *
	 * @param applicationPackage
	 * @return correctly capitalized beginning of a class name (not qualified
	 *         with the package name)
	 */
	private String generateApplicationBaseName(JavaPackage applicationPackage) {
		String pkg = applicationPackage.getFullyQualifiedPackageName();
		return StringUtils.capitalize(pkg.substring(pkg.lastIndexOf(".") + 1));
	}

	public void generateAll(JavaPackage viewPackage, JavaType baseClass) {
		Assert.notNull(pathResolver, "Path resolver required");

		baseClass = prepareCreateEntityViews(baseClass);

		// loop to create all missing views
		FileDetails srcRoot = new FileDetails(new File(
				pathResolver.getRoot(Path.SRC_MAIN_JAVA)), null);
		String antPath = pathResolver.getRoot(Path.SRC_MAIN_JAVA)
				+ File.separatorChar + "**" + File.separatorChar + "*.java";
		SortedSet<FileDetails> entries = fileManager
				.findMatchingAntPath(antPath);

		each_file: for (FileDetails file : entries) {
			String fullPath = srcRoot.getRelativeSegment(file
					.getCanonicalPath());
			// ditch the first / and .java
			fullPath = fullPath.substring(1, fullPath.lastIndexOf(".java"))
					.replace(File.separatorChar, '.');

			JavaType javaType;
			try {
				javaType = new JavaType(fullPath);
			} catch (RuntimeException loopToNextFile) { // ROO-1022
				continue;
			}
			String id = physicalTypeMetadataProvider.findIdentifier(javaType);
			if (id != null) {
				PhysicalTypeMetadata ptm = (PhysicalTypeMetadata) metadataService
						.get(id);
				if (ptm == null
						|| ptm.getPhysicalTypeDetails() == null
						|| !(ptm.getPhysicalTypeDetails() instanceof ClassOrInterfaceTypeDetails)) {
					continue;
				}

				ClassOrInterfaceTypeDetails cid = (ClassOrInterfaceTypeDetails) ptm
						.getPhysicalTypeDetails();
				if (Modifier.isAbstract(cid.getModifier())) {
					continue;
				}

				Set<MetadataItem> metadata = itdMetadataScanner.getMetadata(id);
				for (MetadataItem item : metadata) {
					if (item instanceof EntityMetadata) {
						EntityMetadata em = (EntityMetadata) item;
						Set<String> downstream = dependencyRegistry
								.getDownstream(em.getId());
						// check to see if this entity metadata has an entity
						// view metadata listening to it
						for (String ds : downstream) {
							if (VaadinEntityViewMetadata.isValid(ds)) {
								// there is already a view for this entity
								continue each_file;
							}
						}
						// there is no listening controller, so add one
						JavaType view = new JavaType(
								viewPackage.getFullyQualifiedPackageName()
										+ "." + javaType.getSimpleTypeName()
										+ "View");
						JavaType form = new JavaType(
								viewPackage.getFullyQualifiedPackageName()
										+ "." + javaType.getSimpleTypeName()
										+ "Form");
						JavaType entity = javaType;
						Set<String> disallowedOperations = new HashSet<String>();
						createEntityView(view, form, entity, baseClass,
								disallowedOperations);
						break;
					}
				}
			}
		}
		return;
	}

	private List<String> findValidAbstractEntityViewMids() {
		String providesType = VaadinAbstractEntityViewMetadata
				.getMetadataIdentiferString();
		List<String> allMids = VaadinRooUtils.scanPotentialTypeMids(
				metadataService, fileManager, providesType);

		List<String> entityViewMids = new ArrayList<String>();
		for (String mid : allMids) {
			if (VaadinAbstractEntityViewMetadata.isValid(mid)
					&& metadataService.get(mid) instanceof VaadinAbstractEntityViewMetadata) {
				entityViewMids.add(mid);
			}
		}

		return entityViewMids;
	}

	public void createAutomaticEntityView(JavaType view, JavaType entity,
			JavaType baseClass, Set<String> disallowedOperations) {
		Assert.notNull(pathResolver, "Path resolver required");
		Assert.notNull(classpathOperations, "Class path operations required");

		Assert.notNull(view, "View Java Type required");
		Assert.notNull(entity, "Entity Java Type required");
		Assert.notNull(disallowedOperations,
				"Set of disallowed operations required");

		baseClass = prepareCreateEntityViews(baseClass);
		JavaType form = new JavaType(view.getPackage()
				.getFullyQualifiedPackageName()
				+ "."
				+ entity.getSimpleTypeName() + "Form");
		createEntityView(view, form, entity, baseClass, disallowedOperations);
	}

	/**
	 * Prepare to create entity views: set up necessary dependencies etc.
	 *
	 * Changes (dependencies, helper classes etc.) specific to the use of
	 * JPAContainer are triggered by {@link VaadinAbstractEntityViewMetadata} by
	 * calling {@link #setupJpaContainer()}.
	 *
	 * @param baseClass
	 *            base class (AbstractEntityView) or null to find an unambiguous
	 *            base class that implements {@link RooVaadinAbstractEntityView}
	 * @return base class, looked up based on annotations if not given
	 *
	 * @throws IllegalArgumentException
	 *             if an unambiguous base class is not found or the parameters
	 *             are otherwise incorrect
	 */
	private JavaType prepareCreateEntityViews(JavaType baseClass) {
		Assert.notNull(pathResolver, "Path resolver required");
		Assert.notNull(classpathOperations, "Class path operations required");

		// find base class if not given
		if (baseClass == null) {
			// if no explicitly specified base class, find classes with the
			// @RooVaadinAbstractEntityView annotation using metadata
			List<String> allMids = findValidAbstractEntityViewMids();
			if (allMids.size() == 0) {
				throw new IllegalArgumentException(
						"No entity view base class given and none found in the project.");
			} else if (allMids.size() > 1) {
				throw new IllegalArgumentException(
						"No entity view base class given and multiple candidates found in the project.");
			} else {
				// based on found annotation/metadata
				baseClass = VaadinAbstractEntityViewMetadata
						.getJavaType(allMids.get(0));
			}
		}

		// Parse the configuration.xml file
		Element configuration = XmlUtils.getConfiguration(getClass());
		addBeanValidationDependencies(configuration);

		return baseClass;
	}

	/**
	 * Internal creation of an entity view, not setting up external classes and
	 * dependencies etc.
	 *
	 * @param view
	 *            the view class to create (should not exist)
	 * @param form
	 *            the form class to create (should not exist)
	 * @param entity
	 *            the entity type to manage in the view
	 * @param viewBaseClass
	 *            non-null base class (AbstractEntityView)
	 * @param disallowedOperations
	 * @param useJpaContainer
	 */
	private void createEntityView(JavaType view, JavaType form,
			JavaType entity, JavaType viewBaseClass,
			Set<String> disallowedOperations) {
		Assert.notNull(pathResolver, "Path resolver required");
		Assert.notNull(classpathOperations, "Class path operations required");

		Assert.notNull(view, "View Java Type required");
		Assert.notNull(entity, "Entity Java Type required");
		Assert.notNull(disallowedOperations,
				"Set of disallowed operations required");

		String annotationParameters = "formBackingObject = "
				+ entity.getSimpleTypeName() + ".class";
		for (String operation : disallowedOperations) {
			annotationParameters = annotationParameters + ", " + operation
					+ " = false";
		}

		Map<String, String> substitutions = new HashMap<String, String>();
		substitutions.put("__ENTITY_PACKAGE__", entity.getPackage()
				.getFullyQualifiedPackageName());
		substitutions.put("__ENTITY_CLASS__", entity.getSimpleTypeName());
		substitutions.put("__ABSTRACT_ENTITY_VIEW_PACKAGE__", viewBaseClass
				.getPackage().getFullyQualifiedPackageName());
		substitutions.put("__ABSTRACT_ENTITY_VIEW_CLASS__",
				viewBaseClass.getSimpleTypeName());

		substitutions.put(
				"__ABSTRACT_ENTITY_VIEW_IMPORT__",
				createImport(view, viewBaseClass.getPackage(),
						viewBaseClass.getSimpleTypeName()));
		substitutions.put("__AUTOMATIC_ENTITY_FORM_IMPORT__",
				createImport(view, viewBaseClass.getPackage(),
						AUTOMATIC_ENTITY_FORM_CLASS));
		substitutions.put(
				"__ENTITY_EDITOR_IMPORT__",
				createImport(view, viewBaseClass.getPackage(),
				ENTITY_EDITOR_INTERFACE));

		substitutions.put("__ENTITY_VIEW_PACKAGE__", view.getPackage()
				.getFullyQualifiedPackageName());
		substitutions.put("__ENTITY_VIEW_CLASS__", view.getSimpleTypeName());
		substitutions.put("__ENTITY_VIEW_ANNOTATION_PARAMETERS__",
				annotationParameters);

		String formAnnotationParameters = "formBackingObject = "
			+ entity.getSimpleTypeName() + ".class";

		substitutions.put("__ENTITY_FORM_CLASS__", form.getSimpleTypeName());
		substitutions.put("__ENTITY_FORM_ANNOTATION_PARAMETERS__",
				formAnnotationParameters);

		String resourceIdentifier = classpathOperations
				.getPhysicalLocationCanonicalPath(view, Path.SRC_MAIN_JAVA);

		VaadinRooUtils.installFromTemplateIfNeeded(fileManager,
				resourceIdentifier, "EntityView.java", substitutions);

		String formResourceIdentifier = classpathOperations
				.getPhysicalLocationCanonicalPath(form, Path.SRC_MAIN_JAVA);

		VaadinRooUtils.installFromTemplateIfNeeded(fileManager,
				formResourceIdentifier, "ConcreteAutomaticEntityForm.java",
				substitutions);
	}

	/**
	 * Create an import string for a class in the package of viewBaseClass.
	 *
	 * If importPackage is the same as the package of viewClass, an empty string
	 * is returned.
	 * 
	 * @param viewClass
	 * @param importPackage
	 * @param className
	 * @return
	 */
	private String createImport(JavaType viewClass, JavaPackage importPackage,
			String className) {
		if (!importPackage.equals(viewClass.getPackage())) {
			return "import " + importPackage.getFullyQualifiedPackageName()
					+ "." + className + ";";
		} else {
			return "";
		}
	}

	/**
	 * If necessary, set up the project to enable using JPAContainer: add
	 * dependencies, helper classes etc.
	 *
	 * @param baseClass
	 * @param projectOperations
	 * @param classpathOperations
	 * @param fileManager
	 */
	public static void setupJpaContainer(JavaType baseClass,
			ProjectOperations projectOperations,
			ClasspathOperations classpathOperations, FileManager fileManager) {
		// Parse the configuration.xml file
		Element configuration = XmlUtils
				.getConfiguration(VaadinOperationsImpl.class);
		addJpaContainerDependencies(configuration, projectOperations);
	}

}