package com.vaadin.spring.roo.addon.entityform;

import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import org.springframework.roo.addon.beaninfo.BeanInfoMetadata;
import org.springframework.roo.addon.entity.EntityMetadata;
import org.springframework.roo.addon.plural.PluralMetadata;
import org.springframework.roo.classpath.details.ClassOrInterfaceTypeDetails;
import org.springframework.roo.classpath.details.FieldMetadata;
import org.springframework.roo.classpath.details.ItdTypeDetailsBuilder;
import org.springframework.roo.classpath.details.MethodMetadata;
import org.springframework.roo.classpath.details.MethodMetadataBuilder;
import org.springframework.roo.classpath.itd.InvocableMemberBodyBuilder;
import org.springframework.roo.metadata.MetadataService;
import org.springframework.roo.model.ImportRegistrationResolver;
import org.springframework.roo.model.JavaPackage;
import org.springframework.roo.model.JavaSymbolName;
import org.springframework.roo.model.JavaType;
import org.springframework.roo.project.Path;
import org.springframework.roo.support.util.Assert;

import com.vaadin.spring.roo.addon.VaadinRooUtils;
import com.vaadin.spring.roo.addon.entityview.AbstractVaadinEntityViewMethodBuilder;
import com.vaadin.spring.roo.addon.entityview.ContainerMethodBuilder;
import com.vaadin.spring.roo.addon.entityview.ContainerMethodBuilder.JpaContainerParameters;

public class FormMethodBuilder extends AbstractVaadinEntityViewMethodBuilder {

	private static final String ENTITY_FIELD_WRAPPER_CLASS_SUFFIX = "EntityFieldWrapper";
	private static final String ENTITY_SET_FIELD_WRAPPER_CLASS_SUFFIX = "EntitySetFieldWrapper";

	private static final String ITEM_CLASS = "com.vaadin.data.Item";

	private static final String COMBO_BOX_FIELD_CLASS = "com.vaadin.ui.ComboBox";
	private static final String MULTI_SELECT_FIELD_CLASS = "com.vaadin.ui.TwinColSelect";
	private static final String FORM_FIELD_FACTORY_CLASS = "com.vaadin.ui.FormFieldFactory";
	private static final String DEFAULT_FIELD_FACTORY_CLASS = "com.vaadin.ui.DefaultFieldFactory";
	private static final String FIELD_CLASS = "com.vaadin.ui.Field";
	private static final String TEXT_FIELD_CLASS = "com.vaadin.ui.TextField";
	private static final String DATE_FIELD_CLASS = "com.vaadin.ui.DateField";
	private static final String COMPONENT_CLASS = "com.vaadin.ui.Component";

	private static final String LOCALE_CONTEXT_HOLDER_CLASS = "org.springframework.context.i18n.LocaleContextHolder";

	public static final String GET_FORM_FIELD_FACTORY_METHOD = "getFormFieldFactory";

	private final MetadataService metadataService;
	private final ItdTypeDetailsBuilder builder;

	private JavaPackage abstractViewPackage;

	// map from property name to the underlying (usually entity) type
	private Map<JavaSymbolName, JavaType> specialDomainTypes;

	private Map<JavaType, String> pluralCache;

	private static Logger logger = Logger.getLogger(FormMethodBuilder.class
			.getName());
	private final boolean useJpaContainer;

	public FormMethodBuilder(ClassOrInterfaceTypeDetails governorTypeDetails,
			String metadataId, BeanInfoMetadata beanInfoMetadata,
			EntityMetadata entityMetadata,
			ImportRegistrationResolver importResolver,
			Map<JavaSymbolName, JavaType> specialDomainTypes,
			MetadataService metadataService, ItdTypeDetailsBuilder builder,
			JavaPackage abstractViewPackage, boolean useJpaContainer) {
		super(governorTypeDetails, metadataId, beanInfoMetadata,
				entityMetadata, importResolver);
		this.metadataService = metadataService;
		this.builder = builder;
		this.abstractViewPackage = abstractViewPackage;
		this.useJpaContainer = useJpaContainer;
		this.specialDomainTypes = specialDomainTypes;

		pluralCache = new HashMap<JavaType, String>();
	}

	public List<MethodMetadata> getFormFieldFactoryMethods() {
		List<MethodMetadata> methods = new ArrayList<MethodMetadata>();

		JavaSymbolName methodName = new JavaSymbolName(
				GET_FORM_FIELD_FACTORY_METHOD);

		MethodMetadata method = VaadinRooUtils.methodExists(methodName,
				governorTypeDetails);
		// create sub-methods even if the field factory method exists

		JavaType returnType = new JavaType(FORM_FIELD_FACTORY_CLASS);

		String fieldFactoryClassName = getClassName(DEFAULT_FIELD_FACTORY_CLASS);

		String fieldClassName = getClassName(FIELD_CLASS);
		String itemClassName = getClassName(ITEM_CLASS);
		String componentClassName = getClassName(COMPONENT_CLASS);

		InvocableMemberBodyBuilder bodyBuilder = new InvocableMemberBodyBuilder();
		bodyBuilder.appendFormalLine("return new " + fieldFactoryClassName
				+ "() {");
		bodyBuilder.indent();
		bodyBuilder.appendFormalLine("@Override");
		bodyBuilder.appendFormalLine("public " + fieldClassName
				+ " createField(" + itemClassName
				+ " item, Object propertyId, " + componentClassName
				+ " uiContext) {");
		bodyBuilder.indent();

		methods.addAll(buildCreateFormFieldBody(fieldClassName, bodyBuilder));

		bodyBuilder.indentRemove();
		bodyBuilder.appendFormalLine("}");
		bodyBuilder.indentRemove();
		bodyBuilder.appendFormalLine("};");

		if (method != null) {
			methods.add(method);
		} else {
			MethodMetadataBuilder methodBuilder = new MethodMetadataBuilder(
					getId(), Modifier.PUBLIC, methodName, returnType,
					bodyBuilder);
			methods.add(methodBuilder.build());
		}

		return methods;
	}

	private List<MethodMetadata> buildCreateFormFieldBody(
			String fieldClassName, InvocableMemberBodyBuilder bodyBuilder) {
		List<MethodMetadata> methods = new ArrayList<MethodMetadata>();

		bodyBuilder.appendFormalLine(fieldClassName + " field = null;");

		// enums, collections, domain type references, ... - anything that
		// requires customized field generation
		Map<String, JavaType> fieldSpecialTypes = new HashMap<String, JavaType>();
		for (MethodMetadata accessor : beanInfoMetadata
				.getPublicAccessors(false)) {
			// not interested in identifiers and version fields
			if (accessor.equals(entityMetadata.getIdentifierAccessor())
					|| accessor.equals(entityMetadata.getVersionAccessor())) {
				continue;
			}
			// not interested in fields that are not exposed via a mutator
			FieldMetadata fieldMetadata = beanInfoMetadata
					.getFieldForPropertyName(BeanInfoMetadata
							.getPropertyNameForJavaBeanMethod(accessor));
			if (fieldMetadata == null
					|| !VaadinRooUtils.hasMutator(fieldMetadata,
							beanInfoMetadata)) {
				continue;
			}
			JavaType type = accessor.getReturnType();
			if (VaadinRooUtils.isEnumType(metadataService, type)
					|| isSimpleDomainType(type, fieldMetadata)
					|| isSimpleUnorderedDomainTypeCollection(type,
							fieldMetadata)) {
				fieldSpecialTypes.put(fieldMetadata.getFieldName()
						.getSymbolName(), type);
			}
		}
		boolean haveSpecialTypes = !fieldSpecialTypes.isEmpty();
		if (haveSpecialTypes) {
			boolean first = true;
			for (String property : fieldSpecialTypes.keySet()) {
				bodyBuilder.appendFormalLine((first ? "" : "} else ")
						+ "if (\"" + property + "\".equals(propertyId)) {");
				first = false;
				JavaType type = fieldSpecialTypes.get(property);
				FieldMetadata fieldMetadata = beanInfoMetadata
						.getFieldForPropertyName(new JavaSymbolName(property));
				if (VaadinRooUtils.isEnumType(metadataService, type)
						|| isSimpleDomainType(type, fieldMetadata)) {
					EntityMetadata em = (EntityMetadata) metadataService
							.get(EntityMetadata.createIdentifier(type,
									Path.SRC_MAIN_JAVA));

					bodyBuilder.indent();
					JavaSymbolName methodName = new JavaSymbolName("build"
							+ fieldMetadata.getFieldName()
									.getSymbolNameCapitalisedFirstLetter()
							+ "Combo");
					methods.add(buildDomainTypeComboBoxMethod(type, methodName));
					String selectClassName = getClassName(COMBO_BOX_FIELD_CLASS);
					bodyBuilder.appendFormalLine(selectClassName + " combo = "
							+ methodName.getSymbolName() + "();");

					// import EntityFieldWrapper
					JavaType wrapperType = AbstractVaadinEntityViewMethodBuilder
							.getJavaTypeInWebPackage(abstractViewPackage,
									ENTITY_FIELD_WRAPPER_CLASS_SUFFIX,
									Collections.singletonList(type));
					String wrapperClassName = getClassName(wrapperType);
					String containerMethod = getContainerCreationMethodName(
							type).getSymbolName();
					String entityClassName = getClassName(type);
					bodyBuilder.appendFormalLine("field = new "
							+ wrapperClassName + "(combo, " + entityClassName
							+ ".class, " + containerMethod + "(), \""
							+ em.getIdentifierField().getFieldName() + "\");");

					bodyBuilder
							.appendFormalLine("field.setCaption(createCaptionByPropertyId(propertyId));");
					bodyBuilder.indentRemove();
				} else if (isSimpleUnorderedDomainTypeCollection(type,
						fieldMetadata)) {
					bodyBuilder.indent();
					// first type parameter is the class based on the above if
					JavaType entityType = type.getParameters().get(0);
					EntityMetadata em = (EntityMetadata) metadataService
							.get(EntityMetadata.createIdentifier(entityType,
									Path.SRC_MAIN_JAVA));

					JavaSymbolName methodName = new JavaSymbolName("build"
							+ fieldMetadata.getFieldName()
									.getSymbolNameCapitalisedFirstLetter()
							+ "MultiSelect");
					methods.add(buildDomainTypeMultiSelectMethod(type,
							methodName));
					String selectClassName = getClassName(MULTI_SELECT_FIELD_CLASS);
					bodyBuilder.appendFormalLine(selectClassName + " select = "
							+ methodName.getSymbolName() + "();");

					// import EntitySetFieldWrapper
					JavaType wrapperType = AbstractVaadinEntityViewMethodBuilder
							.getJavaTypeInWebPackage(abstractViewPackage,
									ENTITY_SET_FIELD_WRAPPER_CLASS_SUFFIX,
									Collections.singletonList(entityType));
					String wrapperClassName = getClassName(wrapperType);
					String containerMethod = getContainerCreationMethodName(
							entityType).getSymbolName();
					String entityClassName = getClassName(entityType);
					bodyBuilder.appendFormalLine("field = new "
							+ wrapperClassName + "(select, " + entityClassName
							+ ".class, " + containerMethod + "(), \""
							+ em.getIdentifierField().getFieldName() + "\");");

					bodyBuilder
							.appendFormalLine("field.setCaption(createCaptionByPropertyId(propertyId));");
					bodyBuilder.indentRemove();
				}
			}
			bodyBuilder.appendFormalLine("} else {");
			bodyBuilder.indent();
		}

		// beginning of default branch: use super.createField(...)
		bodyBuilder
				.appendFormalLine("field = super.createField(item, propertyId, uiContext);");

		// default null representation for text fields
		String textFieldClassName = getClassName(TEXT_FIELD_CLASS);
		bodyBuilder.appendFormalLine("if (field instanceof "
				+ textFieldClassName + ") {");
		bodyBuilder.indent();
		bodyBuilder.appendFormalLine("((" + textFieldClassName
				+ ") field).setNullRepresentation(\"\");");
		bodyBuilder.indentRemove();
		bodyBuilder.appendFormalLine("}");

		// locale for date fields
		String dateFieldClassName = getClassName(DATE_FIELD_CLASS);
		String localeContextHolderClassName = getClassName(LOCALE_CONTEXT_HOLDER_CLASS);
		bodyBuilder.appendFormalLine("if (field instanceof "
				+ dateFieldClassName + ") {");
		bodyBuilder.indent();
		bodyBuilder.appendFormalLine("((" + dateFieldClassName
				+ ") field).setLocale(" + localeContextHolderClassName
				+ ".getLocale());");
		bodyBuilder.indentRemove();
		bodyBuilder.appendFormalLine("}");

		// end of default branch: use super.createField(...)

		if (haveSpecialTypes) {
			bodyBuilder.indentRemove();
			bodyBuilder.appendFormalLine("}");
		}

		bodyBuilder.appendFormalLine("return field;");

		return methods;
	}

	private boolean isSimpleDomainType(JavaType type,
			FieldMetadata fieldMetadata) {
		return VaadinRooUtils.isDomainTypeInProject(metadataService, type)
				&& !VaadinRooUtils.isEmbeddedFieldType(fieldMetadata);
	}

	private boolean isSimpleUnorderedDomainTypeCollection(JavaType type,
			FieldMetadata fieldMetadata) {
		logger.fine("Is " + type + " " + fieldMetadata
				+ " a simple unordered domain type collection?");
		if (Set.class.getName().equals(type.getFullyQualifiedTypeName())
				|| HashSet.class.getName().equals(
						type.getFullyQualifiedTypeName())) {
			List<JavaType> parameters = type.getParameters();
			if (parameters != null
					&& parameters.size() == 1
					&& VaadinRooUtils.isDomainTypeInProject(metadataService,
							parameters.get(0))) {
				logger.fine("  Yes");
				return true;
			}
		}
		logger.fine("  No");
		return false;
	}

	private MethodMetadata buildDomainTypeComboBoxMethod(JavaType type,
			JavaSymbolName methodName) {
		MethodMetadata method = VaadinRooUtils.methodExists(methodName,
				governorTypeDetails);
		if (method != null) {
			return method;
		}

		InvocableMemberBodyBuilder bodyBuilder = new InvocableMemberBodyBuilder();

		JavaType comboBoxFieldType = new JavaType(COMBO_BOX_FIELD_CLASS);
		String comboBoxClassName = getClassName(comboBoxFieldType);
		String containerMethod = getContainerCreationMethodName(type)
				.getSymbolName();
		bodyBuilder.appendFormalLine(comboBoxClassName + " combo = new "
				+ comboBoxClassName + "(null, " + containerMethod + "());");
		bodyBuilder.appendFormalLine("Object captionPropertyId = "
				+ getItemCaptionIdMethodName(type).getSymbolName() + "();");
		bodyBuilder.appendFormalLine("if (captionPropertyId != null) {");
		bodyBuilder.indent();
		bodyBuilder
				.appendFormalLine("combo.setItemCaptionPropertyId(captionPropertyId);");
		bodyBuilder.indentRemove();
		bodyBuilder.appendFormalLine("}");
		bodyBuilder.appendFormalLine("return combo;");

		MethodMetadataBuilder methodBuilder = new MethodMetadataBuilder(
				getId(), Modifier.PUBLIC, methodName, comboBoxFieldType,
				bodyBuilder);
		return methodBuilder.build();
	}

	private MethodMetadata buildDomainTypeMultiSelectMethod(JavaType type,
			JavaSymbolName methodName) {
		MethodMetadata method = VaadinRooUtils.methodExists(methodName,
				governorTypeDetails);
		if (method != null) {
			return method;
		}

		InvocableMemberBodyBuilder bodyBuilder = new InvocableMemberBodyBuilder();

		JavaType domainType = type.getParameters().get(0);

		JavaType selectFieldType = new JavaType(MULTI_SELECT_FIELD_CLASS);
		String selectClassName = getClassName(selectFieldType);
		String containerMethod = getContainerCreationMethodName(domainType)
				.getSymbolName();
		bodyBuilder.appendFormalLine(selectClassName + " select = new "
				+ selectClassName + "(null, " + containerMethod + "());");
		bodyBuilder.appendFormalLine("Object captionPropertyId = "
				+ getItemCaptionIdMethodName(domainType).getSymbolName()
				+ "();");
		bodyBuilder.appendFormalLine("if (captionPropertyId != null) {");
		bodyBuilder.indent();
		bodyBuilder
				.appendFormalLine("select.setItemCaptionPropertyId(captionPropertyId);");
		bodyBuilder.indentRemove();
		bodyBuilder.appendFormalLine("}");
		bodyBuilder.appendFormalLine("return select;");

		MethodMetadataBuilder methodBuilder = new MethodMetadataBuilder(
				getId(), Modifier.PUBLIC, methodName, selectFieldType,
				bodyBuilder);
		return methodBuilder.build();
	}

	public List<MethodMetadata> getDomainTypeContainerMethods() {
		List<MethodMetadata> methods = new ArrayList<MethodMetadata>();

		ContainerMethodBuilder containerBuilder = new ContainerMethodBuilder(
				governorTypeDetails, getId(), beanInfoMetadata, entityMetadata,
				builder.getImportRegistrationResolver(), metadataService,
				builder);

		JpaContainerParameters jpaParameters = useJpaContainer ? new JpaContainerParameters(
				true, false) : null;

		for (JavaType type : new LinkedHashSet<JavaType>(
				specialDomainTypes.values())) {
			if (type.equals(beanInfoMetadata.getJavaBean())) {
				continue;
			}

			JavaSymbolName populateMethodName = getContainerCreationMethodName(type);
			MethodMetadata populateMethod = VaadinRooUtils.methodExists(
					populateMethodName, governorTypeDetails);
			if (populateMethod != null) {
				continue;
			}

			// as a side effect, this may create a field for the container
			populateMethod = containerBuilder.buildGetContainerMethod(type,
					populateMethodName, null, jpaParameters);

			if (populateMethod != null) {
				methods.add(populateMethod);
			}

		}
		return methods;
	}

	private JavaSymbolName getContainerCreationMethodName(JavaType type) {
		return new JavaSymbolName("getContainerFor" + getPlural(type));
	}

	private String getPlural(JavaType type) {
		if (pluralCache.get(type) != null) {
			return pluralCache.get(type);
		}
		PluralMetadata pluralMetadata = (PluralMetadata) metadataService
				.get(PluralMetadata.createIdentifier(type, Path.SRC_MAIN_JAVA));
		Assert.notNull(
				pluralMetadata,
				"Could not determine the plural for the '"
						+ type.getFullyQualifiedTypeName() + "' type");
		pluralCache.put(type, pluralMetadata.getPlural());
		return pluralMetadata.getPlural();
	}

}
