package com.vaadin.spring.roo.addon.entityform;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Service;
import org.osgi.service.component.ComponentContext;
import org.springframework.roo.addon.beaninfo.BeanInfoMetadata;
import org.springframework.roo.addon.entity.EntityMetadata;
import org.springframework.roo.classpath.PhysicalTypeIdentifier;
import org.springframework.roo.classpath.PhysicalTypeMetadata;
import org.springframework.roo.classpath.itd.AbstractItdMetadataProvider;
import org.springframework.roo.classpath.itd.ItdTypeDetailsProvidingMetadataItem;
import org.springframework.roo.model.JavaPackage;
import org.springframework.roo.model.JavaType;
import org.springframework.roo.project.Path;

import com.vaadin.spring.roo.addon.VaadinRooUtils;
import com.vaadin.spring.roo.addon.abstractentityview.VaadinAbstractEntityViewMetadata;

/**
 * Provides {@link VaadinAutomaticEntityFormMetadata}, creating ITD with form
 * content and field related methods.
 */
@Component(immediate = true)
@Service
public class VaadinAutomaticEntityFormMetadataProvider extends
		AbstractItdMetadataProvider {

	protected void activate(ComponentContext context) {
		metadataDependencyRegistry.registerDependency(
				PhysicalTypeIdentifier.getMetadataIdentiferType(),
				getProvidesType());
		addMetadataTrigger(VaadinAutomaticEntityFormMetadata
				.getAnnotationType());
	}

	@Override
	protected ItdTypeDetailsProvidingMetadataItem getMetadata(
			String metadataIdentificationString, JavaType aspectName,
			PhysicalTypeMetadata governorPhysicalTypeMetadata,
			String itdFilename) {

		// We know governor type details are non-null and can be safely cast

		// We need to parse the annotation, which we expect to be present
		VaadinAutomaticEntityFormAnnotationValues annotationValues = new VaadinAutomaticEntityFormAnnotationValues(
				governorPhysicalTypeMetadata);
		if (!annotationValues.isAnnotationFound()
				|| annotationValues.formBackingObject == null) {
			return null;
		}

		// Lookup the form backing object's metadata
		JavaType javaType = annotationValues.formBackingObject;
		Path path = Path.SRC_MAIN_JAVA;

		String beanInfoMetadataKey = BeanInfoMetadata.createIdentifier(
				javaType, path);
		String entityMetadataKey = EntityMetadata.createIdentifier(javaType,
				path);

		// We need to lookup the metadata we depend on
		BeanInfoMetadata beanInfoMetadata = (BeanInfoMetadata) metadataService
				.get(beanInfoMetadataKey);
		EntityMetadata entityMetadata = (EntityMetadata) metadataService
				.get(entityMetadataKey);

		// We need to abort if we couldn't find dependent metadata
		if (beanInfoMetadata == null || !beanInfoMetadata.isValid()
				|| entityMetadata == null || !entityMetadata.isValid()) {
			return null;
		}

		// We need to be informed if our dependent metadata changes
		metadataDependencyRegistry.registerDependency(beanInfoMetadataKey,
				metadataIdentificationString);
		metadataDependencyRegistry.registerDependency(entityMetadataKey,
				metadataIdentificationString);

		// We also need to be informed if a referenced type is changed
		Collection<JavaType> specialDomainTypes = VaadinRooUtils
				.getSpecialDomainTypes(metadataService,
						beanInfoMetadata.getJavaBean(), true).values();
		// unique types only
		for (JavaType type : new LinkedHashSet<JavaType>(specialDomainTypes)) {
			metadataDependencyRegistry.registerDependency(
					BeanInfoMetadata.createIdentifier(type, path),
					metadataIdentificationString);
		}

		// find any class with the annotation @RooVaadinAbstractEntityView
		// same package will contain helper classes

		// TODO this is not the best way - could pick the wrong one if multiple
		// exist etc. - and depends on such a class existing

		VaadinAbstractEntityViewMetadata abstractViewMetadata = null;

		String providesType = VaadinAbstractEntityViewMetadata
				.getMetadataIdentiferString();
		List<String> allMids = VaadinRooUtils.scanPotentialTypeMids(
				metadataService, fileManager, providesType);
		for (String mid : allMids) {
			if (VaadinAbstractEntityViewMetadata.isValid(mid)) {
				abstractViewMetadata = (VaadinAbstractEntityViewMetadata) metadataService
						.get(mid);
				if (abstractViewMetadata != null) {
					break;
				}
			}
		}

		// must have the package and JPAContainer use information
		if (abstractViewMetadata == null) {
			return null;
		}

		JavaType parentType = VaadinAbstractEntityViewMetadata
				.getJavaType(abstractViewMetadata.getId());
		JavaPackage abstractViewPackage = parentType.getPackage();
		boolean useJpaContainer = abstractViewMetadata.isUseJpaContainer();

		return new VaadinAutomaticEntityFormMetadata(
				metadataIdentificationString, aspectName,
				governorPhysicalTypeMetadata, metadataService,
				annotationValues, beanInfoMetadata, entityMetadata,
				abstractViewPackage, useJpaContainer);
	}

	public String getItdUniquenessFilenameSuffix() {
		return "VaadinAutomaticEntityForm";
	}

	@Override
	protected String getGovernorPhysicalTypeIdentifier(
			String metadataIdentificationString) {
		JavaType javaType = VaadinAutomaticEntityFormMetadata
				.getJavaType(metadataIdentificationString);
		Path path = VaadinAutomaticEntityFormMetadata
				.getPath(metadataIdentificationString);
		String physicalTypeIdentifier = PhysicalTypeIdentifier
				.createIdentifier(javaType, path);
		return physicalTypeIdentifier;
	}

	@Override
	protected String createLocalIdentifier(JavaType javaType, Path path) {
		return VaadinAutomaticEntityFormMetadata.createIdentifier(javaType,
				path);
	}

	public String getProvidesType() {
		return VaadinAutomaticEntityFormMetadata.getMetadataIdentiferType();
	}

}
